@@ -14,10 +14,10 @@ int32_t SPVM__Net__SSLeay__OPENSSL_INIT_SETTINGS__DESTROY(SPVM_ENV* env, SPVM_VA
   
   void* obj_self = stack[0].oval;
   
-  OPENSSL_INIT_SETTINGS* pointer = env->get_pointer(env, stack, obj_self);
+  OPENSSL_INIT_SETTINGS* self = env->get_pointer(env, stack, obj_self);
   
   if (!env->no_free(env, stack, obj_self)) {
-    OPENSSL_INIT_free(pointer);
+    OPENSSL_INIT_free(self);
   }
   
   return 0;
